import asyncio
from gettext import gettext as _
from pathlib import Path
from typing import TypedDict

from textual.app import ComposeResult
from textual.binding import Binding
from textual.containers import ScrollableContainer
from textual.screen import ModalScreen
from textual.suggester import SuggestFromList
from textual.validation import Regex
from textual.widgets import Checkbox, Footer, Header, Input, ProgressBar, Select
from textual.widgets import Static as Placeholder

from ..log import Logger
from ..types import POFileHandler, TranslationServices
from ..utils import NotifyException, apply_styles, correct_translation, wait_for_element


class TranslationServiceConfig(TypedDict, total=False):
    source: str
    target: str
    api_key: str | None
    proxies: dict[str, str] | None
    model: str | None
    region: str | None
    api_key_type: str | None


class Translator(ModalScreen[None], POFileHandler):
    """A modal screen for translating PO files using Google Translate."""

    BINDINGS = [
        Binding(key="t", action="translate", description=_("Translate"), show=True),
        Binding(key="q", action="quit", description=_("Quit"), show=True),
        Binding(key="o", action="toggle_override", description=_("Toggle Override Existing"), show=True),
    ]

    def __init__(self, po_path: Path, translation_config: TranslationServiceConfig):
        """Initialize the Translator modal.

        Args:
            po_path (Path): Path to the PO file to be translated.
            translation_service (TranslationServices): The translation service to use.
            config (TranslationServiceConfig): Configuration for the translation service.
        """
        ModalScreen.__init__(self)  # pyright: ignore[reportUnknownMemberType]

        self._translating = False
        self._translation_config = translation_config

        self.logger.info(
            "Loading PO file for translation...",
            extra={"context": "Translator.init", "path": str(po_path)},
        )
        POFileHandler.__init__(self, po_path)
        self.logger.info(
            "PO file loaded",
            extra={
                "context": "Translator.init",
                "path": str(self.pofile_path),
                "entries": len(list(self.pofile)),  # pyright: ignore[reportUnknownArgumentType, reportArgumentType]
            },
        )

    @property
    def logger(self) -> Logger:
        """Return the application logger."""
        return getattr(
            self.app,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
            "logger",
        )

    def compose_proxies(self) -> ComposeResult:
        """Compose the proxy input fields."""
        yield Input(
            placeholder=_("HTTP Proxy"),
            value=((self._translation_config.get("proxies", {}) or {}).get("http", "")),
            name="proxy_http",
            suggester=SuggestFromList(["http://"]),
        )
        yield Input(
            placeholder=_("HTTPS Proxy"),
            value=((self._translation_config.get("proxies", {}) or {}).get("https", "")),
            name="proxy_https",
            suggester=SuggestFromList(["https://"]),
        )

    def compose_model(self) -> ComposeResult:
        """Compose the model input field if supported."""
        yield Input(placeholder=_("Model"), value=self._translation_config.get("model") or "", name="model")

    def compose_region(self) -> ComposeResult:
        """Compose the region input field if supported."""
        yield Input(placeholder=_("Region"), value=self._translation_config.get("region") or "", name="region")

    def compose_api_key(self) -> ComposeResult:
        """Compose the API key input field if needed."""
        yield Input(
            placeholder=_("API Key"), value=self._translation_config.get("api_key") or "", password=True, name="api_key"
        )
        yield Input(
            placeholder=_("API Key Type (free or paid)"),
            value=self._translation_config.get("api_key_type") or "",
            name="api_key_type",
            suggester=SuggestFromList(["free", "paid"]),
            validators=(Regex(r"^(free|paid)$", flags=0, failure_description=_("Must be 'free' or 'paid'")),),
            validate_on=("submitted", "changed"),
        )

    def compose(self) -> ComposeResult:
        """Compose the UI elements for the modal."""
        yield Header()
        yield apply_styles(
            ScrollableContainer(
                apply_styles(Checkbox(label=_("Override existing translations"), value=False), width="1fr"),
                apply_styles(
                    Select(
                        ((s.value, s.value) for s in TranslationServices),
                        value=TranslationServices.GOOGLE_TRANSLATE.value,
                        id="translation_service",
                        prompt=_("Select Translation Service"),
                    ),
                    width="1fr",
                ),
                apply_styles(ScrollableContainer(*self.compose_proxies(), id="translator_settings"), width="1fr"),
                apply_styles(Placeholder(), height="1fr"),
                apply_styles(
                    ProgressBar(
                        total=sum(
                            1 + (1 if e.msgid_plural else 0)  # pyright: ignore[reportUnknownMemberType]
                            for e in self.pofile  # pyright: ignore[reportUnknownVariableType]
                        )
                    ),
                    vertical="bottom",
                    width="1fr",
                ),
            ),
            vertical="top",
            width="1fr",
        )
        yield Footer()

    async def apply_translation_settings(self):
        """Apply the translation settings from the input fields."""
        settings_container = await wait_for_element(lambda: self.query_one("#translator_settings", ScrollableContainer))
        inputs = settings_container.query(Input)
        for input_widget in inputs:
            match input_widget.name:
                case "region" | "model" | "api_key" | "api_key_type":
                    self._translation_config[input_widget.name] = input_widget.value or None

                case "proxy_http" | "proxy_https":
                    proxies = self._translation_config.get("proxies") or {}
                    if input_widget.value:
                        proxies[input_widget.name.removeprefix("proxy_")] = input_widget.value
                    else:
                        proxies.pop(input_widget.name.removeprefix("proxy_"), None)
                    self._translation_config["proxies"] = proxies or None

                case _:
                    pass

        await asyncio.sleep(0)

    async def on_select_changed(self, event: Select.Changed):
        """Handle changes in the translation service selection."""
        if self._translating:
            self.logger.warning(
                "Translation service change ignored during active translation",
                extra={"context": "Translator.on_select_changed"},
            )
            return

        if event.value is Select.BLANK:
            self.logger.warning(
                "Blank translation service selected, ignoring", extra={"context": "Translator.on_select_changed"}
            )
            return

        settings_container = await wait_for_element(lambda: self.query_one("#translator_settings", ScrollableContainer))
        selected_service = TranslationServices(event.value).translation_service_protocol
        await settings_container.remove_children()
        if selected_service.needs_api_key():
            await settings_container.mount_all(self.compose_api_key())
        if selected_service.supports_model():
            await settings_container.mount_all(self.compose_model())
        if selected_service.supports_region():
            await settings_container.mount_all(self.compose_region())
        if selected_service.supports_proxies():
            await settings_container.mount_all(self.compose_proxies())

        self.logger.info(
            "Translation service changed",
            extra={"context": "Translator.on_select_changed", "service": event.value},
        )
        await asyncio.sleep(0)

    async def action_toggle_override(self):
        """Toggle the override existing translations checkbox."""
        if self._translating:
            self.logger.warning(
                "Override toggle ignored during active translation",
                extra={"context": "Translator.action_toggle_override"},
            )
            return

        self.logger.debug(
            "Toggling override existing translations", extra={"context": "Translator.action_toggle_override"}
        )
        checkbox = await wait_for_element(lambda: self.query_one(Checkbox))
        checkbox.value = not checkbox.value
        self.logger.info(
            "Override existing translations set to",
            extra={"value": checkbox.value, "context": "Translator.action_toggle_override"},
        )

    async def action_quit(self):
        """Quit the modal."""
        self.logger.info("Quitting Translator modal", extra={"context": "Translator.action_quit"})
        self.dismiss()
        self.logger.info("Translator modal dismissed", extra={"context": "Translator.action_quit"})

    async def action_translate(self):
        """Start the translation process."""
        self.logger.info("Starting translation process", extra={"context": "Translator.action_translate"})

        self.run_worker(self.translate_pofile, group="translation")
        self.logger.info("Translation process started", extra={"context": "Translator.action_translate"})

    async def translate_pofile(self):
        """Translate the PO file using Google Translate."""
        self.logger.info("Translating PO file...", extra={"context": "Translator.translate_po"})

        self._translating = True
        progressbar = await wait_for_element(lambda: self.query_one(ProgressBar))
        override_existing = (await wait_for_element(lambda: self.query_one(Checkbox))).value
        selected_value = (await wait_for_element(self.query_one, selector=Select)).value

        selected_service = (
            TranslationServices(selected_value)
            if selected_value is not Select.BLANK
            else TranslationServices.GOOGLE_TRANSLATE
        )
        await self.apply_translation_settings()
        self.logger.info(
            "Using translation service",
            extra={
                "context": "Translator.translate_po",
                "service": selected_service.value,
                "config": self._translation_config,
            },
        )
        with NotifyException(self):
            translator = selected_service.translation_service_protocol(self._translation_config)  # type: ignore[reportArgumentType]

            self.notify(
                _("Translating PO file... This may take a while depending on the file size."),
                timeout=5,
                title=_("⏳ Translation Started"),
            )

            for (
                idx,
                entry,  # pyright: ignore[reportUnknownVariableType]
            ) in enumerate(self.pofile):
                if entry.msgid_plural:  # pyright: ignore[reportUnknownMemberType]
                    if override_existing or not all(
                        entry.msgstr_plural.values()  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                    ):
                        entry.msgstr_plural = {
                            index: correct_translation(
                                elem,  # type: ignore[reportUnknownArgumentType]
                                (
                                    await translator.translate(
                                        elem,  # type: ignore[reportUnknownArgumentType]
                                    )
                                ),
                            )
                            for index, elem in enumerate(  # pyright: ignore[reportUnknownVariableType]
                                (
                                    entry.msgid,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                    entry.msgid_plural,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                )
                            )
                        }
                        self.logger.debug(
                            "Translated plural entry",
                            extra={
                                "msgid": entry.msgid,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                "msgid_plural": entry.msgid_plural,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                "context": "Translator.translate_po",
                            },
                        )
                        self.pofile[idx] = entry
                    progressbar.advance(2)
                    await asyncio.sleep(0)

                elif entry.msgid:  # pyright: ignore[reportUnknownMemberType]
                    if override_existing or not entry.msgstr:  # pyright: ignore[reportUnknownMemberType]
                        entry.msgstr = correct_translation(
                            entry.msgid,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                            (
                                await translator.translate(  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType, reportArgumentType]
                                    entry.msgid,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                )
                            ),
                        )
                        self.logger.debug(
                            "Translated singular entry",
                            extra={
                                "msgid": entry.msgid,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                "msgstr": entry.msgstr,  # pyright: ignore[reportUnknownMemberType, reportUnknownArgumentType]
                                "context": "Translator.translate_po",
                            },
                        )
                        self.pofile[idx] = entry
                    progressbar.advance(1)
                    await asyncio.sleep(0)

            self.logger.info(
                "Translation completed, saving PO file...",
                extra={"context": "Translator.translate_po", "path": str(self.pofile_path)},
            )
            self.pofile.save(str(self.pofile_path))  # pyright: ignore[reportUnknownMemberType]
            self._translating = False
            self.dismiss()
            self.logger.info(
                "PO file saved and modal dismissed",
                extra={"context": "Translator.translate_po", "path": str(self.pofile_path)},
            )
            self.notify(_("Translation completed and PO file saved."), timeout=5, title=_("⌛ Translation Completed"))



from enum import Enum
from gettext import gettext as _
from pathlib import Path
from typing import Any, Dict, NamedTuple, Protocol, Tuple, Type, TypedDict

import polib
from babel import negotiate_locale
from deep_translator import (  # pyright: ignore[reportMissingTypeStubs]
    ChatGptTranslator,
    DeeplTranslator,
    GoogleTranslator,
    MicrosoftTranslator,
    MyMemoryTranslator,
    YandexTranslator,
)
from deep_translator.constants import (  # pyright: ignore[reportMissingTypeStubs]
    DEEPL_LANGUAGE_TO_CODE,
    GOOGLE_LANGUAGES_TO_CODES,
    MY_MEMORY_LANGUAGES_TO_CODES,
)


class TranslationServiceConfig(TypedDict):
    source: str
    target: str
    api_key: str | None
    api_key_type: str | None
    proxies: dict[str, str] | None
    model: str | None
    region: str | None


class DeeplTranslationService(DeeplTranslator):
    def __init__(self, config: TranslationServiceConfig):
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=negotiate_locale([config["source"]], DEEPL_LANGUAGE_TO_CODE.values()) or config["source"],
            target=negotiate_locale([config["target"]], DEEPL_LANGUAGE_TO_CODE.values()) or config["target"],
            api_key=config["api_key"],
            use_free_api=config.get("api_key_type", "free") == "free",
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return True

    @classmethod
    def supports_model(cls) -> bool:
        return False

    @classmethod
    def supports_region(cls) -> bool:
        return False

    @classmethod
    def supports_proxies(cls) -> bool:
        return False

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        return super().translate(text)  # pyright: ignore[reportUnknownMemberType]


class GoogleTranslationService(GoogleTranslator):
    def __init__(self, config: TranslationServiceConfig):
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=negotiate_locale([config["source"]], GOOGLE_LANGUAGES_TO_CODES.values()) or config["source"],
            target=negotiate_locale([config["target"]], GOOGLE_LANGUAGES_TO_CODES.values()) or config["target"],
            proxies=config["proxies"],
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return False

    @classmethod
    def supports_model(cls) -> bool:
        return False

    @classmethod
    def supports_region(cls) -> bool:
        return False

    @classmethod
    def supports_proxies(cls) -> bool:
        return True

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        return super().translate(text)  # pyright: ignore[reportUnknownMemberType]


class MyMemoryTranslationService(MyMemoryTranslator):
    def __init__(self, config: TranslationServiceConfig):
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=negotiate_locale(
                [config["source"]],
                MY_MEMORY_LANGUAGES_TO_CODES.values(),
                sep="-",
            )
            or config["source"],
            target=negotiate_locale(
                [config["target"]],
                MY_MEMORY_LANGUAGES_TO_CODES.values(),
                sep="-",
            )
            or config["target"],
            proxies=config["proxies"],
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return False

    @classmethod
    def supports_proxies(cls) -> bool:
        return True

    @classmethod
    def supports_model(cls) -> bool:
        return False

    @classmethod
    def supports_region(cls) -> bool:
        return False

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        result = super().translate(text)  # pyright: ignore[reportUnknownMemberType]
        return " ".join(result).replace("  ", " ") if isinstance(result, list) else result


class MicrosoftTranslationService(MicrosoftTranslator):
    def __init__(self, config: TranslationServiceConfig):
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=config["source"],
            target=config["target"],
            api_key=config["api_key"],
            region=config["region"],
            proxies=config["proxies"],
        )
        # TODO: evaluate and store as constant type
        MICROSOFT_LANGUAGES_TO_CODES = self.get_supported_languages(  # pyright: ignore[reportUnknownVariableType, reportUnknownMemberType] # keep at one line
            as_dict=True
        )
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=negotiate_locale(
                [config["source"]],
                MICROSOFT_LANGUAGES_TO_CODES.values(),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            )
            or config["source"],
            target=negotiate_locale(
                [config["target"]],
                MICROSOFT_LANGUAGES_TO_CODES.values(),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            )
            or config["target"],
            api_key=config["api_key"],
            region=config["region"],
            proxies=config["proxies"],
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return True

    @classmethod
    def supports_model(cls) -> bool:
        return False

    @classmethod
    def supports_region(cls) -> bool:
        return True

    @classmethod
    def supports_proxies(cls) -> bool:
        return True

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        return super().translate(text)  # pyright: ignore[reportUnknownMemberType]


class YandexTranslationService(YandexTranslator):
    def __init__(self, config: TranslationServiceConfig):
        self._proxies = config["proxies"]
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=config["source"], target=config["target"], api_key=config["api_key"]
        )
        # TODO: evaluate and store as constant type
        YANDEX_LANGUAGES_TO_CODES = self.get_supported_languages(  # pyright: ignore[reportUnknownVariableType, reportUnknownMemberType] # keep at one line
            as_dict=True
        )
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=negotiate_locale(
                [config["source"]],
                YANDEX_LANGUAGES_TO_CODES.values(),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            )
            or config["source"],
            target=negotiate_locale(
                [config["target"]],
                YANDEX_LANGUAGES_TO_CODES.values(),  # pyright: ignore[reportUnknownArgumentType, reportUnknownMemberType, reportAttributeAccessIssue]
            )
            or config["target"],
            api_key=config["api_key"],
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return True

    @classmethod
    def supports_model(cls) -> bool:
        return False

    @classmethod
    def supports_region(cls) -> bool:
        return False

    @classmethod
    def supports_proxies(cls) -> bool:
        return True

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        return super().translate(text, proxies=self._proxies)  # pyright: ignore[reportUnknownMemberType]


class ChatGPTTranslationService(ChatGptTranslator):
    def __init__(self, config: TranslationServiceConfig):
        super().__init__(  # pyright: ignore[reportUnknownMemberType]
            source=config["source"], target=config["target"], api_key=config["api_key"], model=config["model"]
        )

    @classmethod
    def needs_api_key(cls) -> bool:
        return True

    @classmethod
    def supports_model(cls) -> bool:
        return True

    @classmethod
    def supports_region(cls) -> bool:
        return False

    @classmethod
    def supports_proxies(cls) -> bool:
        return False

    async def translate(self, text: str) -> str:  # pyright: ignore[reportIncompatibleMethodOverride]
        return super().translate(text)  # pyright: ignore[reportUnknownMemberType]


class TranslationServiceProtocol(Protocol):
    def __init__(self, config: TranslationServiceConfig): ...
    @classmethod
    def needs_api_key(cls) -> bool: ...
    @classmethod
    def supports_model(cls) -> bool: ...
    @classmethod
    def supports_region(cls) -> bool: ...
    @classmethod
    def supports_proxies(cls) -> bool: ...
    async def translate(self, text: str) -> str: ...


class TranslationServices(str, Enum):
    GOOGLE_TRANSLATE = _("Google Translate")
    MY_MEMORY = _("MyMemory Translator")
    MICROSOFT_TRANSLATE = _("Microsoft Translator")
    YANDEX_TRANSLATE = _("Yandex Translate")
    CHATGPT = _("ChatGPT Translation Service")
    DEEPL_TRANSLATOR = _("DeepL Translator")

    @property
    def translation_service_protocol(self) -> Type[TranslationServiceProtocol]:
        match self:
            case TranslationServices.GOOGLE_TRANSLATE:
                return GoogleTranslationService
            case TranslationServices.MY_MEMORY:
                return MyMemoryTranslationService
            case TranslationServices.MICROSOFT_TRANSLATE:
                return MicrosoftTranslationService
            case TranslationServices.YANDEX_TRANSLATE:
                return YandexTranslationService
            case TranslationServices.CHATGPT:
                return ChatGPTTranslationService
            case TranslationServices.DEEPL_TRANSLATOR:
                return DeeplTranslationService
            case _:
                raise NotImplementedError(f"Translation service {self.value} is not implemented.")


class POFileHandler:
    def __init__(self, po_path: Path):
        """Initialize the POFileHandler with the given PO file path.

        Args:
            po_path (Path): Path to the PO file.
        """
        self.pofile_path = po_path
        self.pofile = po_path  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]

    @property
    def pofile_path(self) -> Path:
        """Path to the PO file."""
        return self._po_path

    @pofile_path.setter
    def pofile_path(self, value: Path):
        """Set the PO file path."""
        self._po_path = value

    @property
    def pofile(self) -> polib.POFile:
        """Return the loaded PO file."""
        return self._po  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType, reportReturnType]

    @pofile.setter
    def pofile(self, value: polib.POFile | Path) -> None:
        """Set the PO file."""
        if isinstance(value, Path):
            file = polib.pofile(str(value))  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
            if isinstance(file, polib.POFile):  # pyright: ignore[reportUnnecessaryIsInstance]
                self._po = file
            else:
                raise ValueError(f"Failed to load PO file from path: {value}")
        else:
            self._po = value


class SubCommand(NamedTuple):
    """A sub-command of the application."""

    name: str
    description: str
    requires_ui: bool
    default_check: bool

    @property
    def selection_list_item(self) -> Tuple[str, str, bool]:
        """Return a tuple suitable for use in a SelectionList item."""
        return (self.description, self.name, self.default_check)


class TableCell(NamedTuple):
    """A cell in the PO review table."""

    row_no: str
    type: str
    msgid: str
    msgstr: str


class SingletonType(type):
    """A metaclass for singleton classes."""

    _instances: Dict[str, object] = {}

    def __new__(cls, name: str, bases: Tuple[type, ...], attrs: Dict[str, Any]) -> Any:
        if name not in cls._instances:
            instance = super().__new__(cls, name, bases, attrs)
            cls._instances[name] = instance
        return cls._instances[name]



import argparse
import sys
import tomllib
from gettext import gettext as _
from importlib.metadata import version as pkg_version
from pathlib import Path
from typing import Any, Dict, List, Literal, Optional

import jmespath
from pydantic import BaseModel, Field

from .log import Logger
from .types import TranslationServiceConfig
from .utils import get_base_type

logger = Logger()


class Config(BaseModel):
    """unbabelizer loads configuration from pyproject.toml and command line arguments."""

    author: str = Field(
        description=_("Author of the project"),
        json_schema_extra={"pyproject.toml": "project.authors[0].name", "argparse.flag": "--author"},
    )
    email: str = Field(
        description=_("Email of the author"),
        json_schema_extra={"pyproject.toml": "project.authors[0].email", "argparse.flag": "--email"},
    )
    version: str = Field(
        description=_("Version of the project"),
        json_schema_extra={"pyproject.toml": "project.version", "argparse.flag": "--version"},
    )
    title: str = Field(
        description=_("Title of the project"),
        json_schema_extra={"pyproject.toml": "project.name", "argparse.flag": "--title"},
    )
    locale_dir: Path = Field(
        default=Path("locale"),
        description=_("Directory for locale files"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.locale_dir", "argparse.flag": "--locale-dir"},
    )
    input_paths: List[Path] = Field(
        default=[Path.cwd()],
        description=_("Paths to search for source files"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.input_paths", "argparse.flag": "--input-paths"},
    )
    exclude_patterns: List[str] = Field(
        default=[],
        description=_("Paths to exclude from searching"),
        json_schema_extra={
            "pyproject.toml": "tool.unbabelizer.exclude_patterns",
            "argparse.flag": "--exclude-patterns",
        },
    )
    src_lang: str = Field(
        default="en",
        description=_("Source language code"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.src_lang", "argparse.flag": "--src-lang"},
    )
    dest_lang: List[str] = Field(
        description=_("Destination language code"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.dest_lang", "argparse.flag": "--dest-lang"},
    )
    domain: str = Field(
        default="messages",
        description=_("Domain for the .po files"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.domain", "argparse.flag": "--domain"},
    )
    mapping_file: str = Field(
        default="[python: **.py]\nencoding = utf-8\n",
        description=_("Path to the Babel mapping file (as CLI argument) or its content (in pyproject.toml config)"),
        json_schema_extra={
            "pyproject.toml": "tool.unbabelizer.mapping_file_content",
            "argparse.flag": "--mapping-file",
        },
    )
    line_width: int = Field(
        default=120,
        description=_("Line width for .po files"),
        json_schema_extra={
            "pyproject.toml": "tool.unbabelizer.line_width",
            "argparse.flag": "--line-width",
        },
    )
    keywords: List[str] = Field(
        default=[],
        description=_("Additional keywords to look for in source files"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.keywords", "argparse.flag": "--keywords"},
    )
    http_proxy: Optional[str] = Field(
        default=None,
        description=_("HTTP proxy URL"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.proxy_http", "argparse.flag": "--http-proxy"},
    )
    https_proxy: Optional[str] = Field(
        default=None,
        description=_("HTTPS proxy URL"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.proxy_https", "argparse.flag": "--https-proxy"},
    )
    api_key: Optional[str] = Field(
        default=None,
        description=_("API key for the translation service, if required"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.api_key", "argparse.flag": "--api-key"},
    )
    api_key_type: Optional[Literal["free", "paid"]] = Field(
        default=None,
        description=_('Type of API key, if applicable (either "free" or "paid")'),
        json_schema_extra={
            "pyproject.toml": "tool.unbabelizer.api_key_type",
            "argparse.flag": "--api-key-type",
            "argparse.choices": ["free", "paid"],
        },
        pattern="^(free|paid)$",
    )
    model: Optional[str] = Field(
        default=None,
        description=_("Model to use for the translation service, if applicable"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.model", "argparse.flag": "--model"},
    )
    region: Optional[str] = Field(
        default=None,
        description=_("Region for the translation service, if applicable"),
        json_schema_extra={"pyproject.toml": "tool.unbabelizer.region", "argparse.flag": "--region"},
    )

    def get_translation_config(self, dest_lang_index: int) -> TranslationServiceConfig:
        """Prepare the configuration dictionary for the translation service."""
        return {
            "source": self.src_lang,
            "target": self.dest_lang[dest_lang_index],
            "api_key": self.api_key,
            "api_key_type": self.api_key_type,
            "proxies": {
                k: v
                for k, v in {
                    "http": self.http_proxy,
                    "https": self.https_proxy,
                }.items()
                if v is not None
            }
            or None,
            "model": self.model,
            "region": self.region,
        }

    @classmethod
    def source_cli_args(cls, args: List[str]) -> Dict[str, Any]:
        """Parse command line arguments to create a Config instance."""
        parser = argparse.ArgumentParser(
            description=_("unbabelizer {version} configuration").format(version=pkg_version("unbabelizer"))
        )
        for name, field in cls.model_fields.items():
            schema = (  # pyright: ignore[reportUnknownVariableType]
                field.json_schema_extra  # pyright: ignore[reportUnknownMemberType]
                if isinstance(field.json_schema_extra, dict)
                else {}
            )

            flag = schema.get(  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
                "argparse.flag", f"--{name.replace('_', '-')}"
            )
            choices = schema.get(  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
                "argparse.choices", None
            )
            config_field = schema.get(  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
                "pyproject.toml", name
            )
            if choices is not None and not isinstance(choices, list):
                choices = [choices]  # pyright: ignore[reportUnknownVariableType]
            parser.add_argument(
                flag,  # pyright: ignore[reportArgumentType]
                help=(field.description or "")
                + " "
                + _('(overrides pyproject.toml setting: "{config_field}")').format(
                    config_field=config_field  # pyright: ignore[reportUnknownArgumentType]
                ),
                default=None,
                nargs=None if field.annotation is not list else "+",
                type=get_base_type(field.annotation),
                choices=choices,  # pyright: ignore[reportUnknownArgumentType]
            )

        parsed_args = parser.parse_args(args)
        field_values = {k: v for k, v in vars(parsed_args).items() if v is not None}
        logger.debug("Parsed CLI args", extra={"context": "Config.source_cli_args", "data": field_values})
        return field_values

    @classmethod
    def source_pyproject_toml(cls, file_path: Path) -> Dict[str, Any]:
        """Load configuration from pyproject.toml using jmespath."""
        if not file_path.exists() or not file_path.is_file():
            return {}

        pyproject_data = tomllib.loads(file_path.read_text())
        logger.debug(
            "Loaded pyproject.toml data:", extra={"context": "Config.source_pyproject_toml", "data": pyproject_data}
        )
        field_values: Dict[str, Any] = {}
        for name, field in cls.model_fields.items():
            schema = (  # pyright: ignore[reportUnknownVariableType]
                field.json_schema_extra  # pyright: ignore[reportUnknownMemberType]
                if isinstance(field.json_schema_extra, dict)
                else {}
            )
            jmespath_query = schema.get(  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
                "pyproject.toml", name
            )
            value = jmespath.search(  # pyright: ignore[reportUnknownMemberType, reportUnknownVariableType]
                jmespath_query, pyproject_data  # pyright: ignore[reportArgumentType]c
            )
            logger.debug(
                "JMESPath query result",
                extra={  # pyright: ignore[reportUnknownArgumentType]
                    "context": "Config.source_pyproject_toml",
                    "field": name,
                    "query": jmespath_query,
                    "value": value,
                },
            )
            field_values[name] = value

        field_values = {k: v for k, v in field_values.items() if v is not None}
        logger.debug(
            "Extracted config data from pyproject.toml",
            extra={"context": "Config.source_pyproject_toml", "data": field_values},
        )
        return field_values

    @classmethod
    def build(cls, args: List[str] | None) -> "Config":
        """Validate and create a Config instance from source data."""
        config_data = cls.source_pyproject_toml(Path("pyproject.toml"))
        config_data.update(cls.source_cli_args(args or sys.argv[1:]))
        logger.info("Validated config data", extra={"context": "Config.build", "data": config_data})
        return cls.model_validate(config_data)
